#!/usr/bin/perl
# 
# Directors.pm: Routines to load and use the directors database.
#
# Copyright (c) Warren Toomey, wkt@tuhs.org, 2002/2003/2004, GPL license
#
#    $Revision: 1.3 $
#
# This file is distributed under the GNU Public License with the
# following additional limitations.
# 
# 1. This software must not be used in those parts of the world where
#    TiVo Inc. provides a subscription service.
# 2. This software must not be used for commercial purposes. You may 
#    not use this software to provide a "service" and charge for that
#    service.
#
# See Docs/COPYING for the GNU Public License details.
#
package TiVo::Directors;
use strict;
use TiVo::ConfigFiles;
use TiVo::MiscUtil;

BEGIN {
  use Exporter	 ();
  use vars	 qw($VERSION @ISA @EXPORT @EXPORT_OK %EXPORT_TAGS);

  $VERSION = do { my @r = (q$Revision: 1.3 $ =~ /\d+/g); sprintf "%d."."%02d" x $#r, @r }; # must be all one line, for MakeMaker

  @ISA	       = qw(Exporter);
  @EXPORT      = qw(Read_Directors Find_Director);
  %EXPORT_TAGS = ();
  @EXPORT_OK   = qw();
}

use vars	 @EXPORT_OK;
END { }	 # module clean-up code here (global destructor)


#####################################
###### Package-private Globals ######
#####################################

my ($directors_file)= "directors.tivo.gz";
my (%Directors, %YearDirectors);

######################
###### Routines ######
######################

# Read_Directors: read the directors from the directors.tivo.gz file.
sub Read_Directors
{
  my ($Configdir);
  my ($title,$year,$director);

  # Don't do anything if there is no directors file
  $Configdir= $ConfigVar{"Configdir"};
  if (! -f "$Configdir/$directors_file") {
    Debug("No directors file $Configdir/$directors_file!\n");
    return(1);
  }

  print("Loading directors file...\n");
  unless (open(RIN, "zcat $Configdir/$directors_file |")) {
    print(STDERR "Cannot open $Configdir/$directors_file: $!\n");
    return(0);
  }

  # Walk the file obtaining per-movie directors
  while (<RIN>) {
   chop;
   ($title,$director,$year)= split(/\t/);
   if (defined($year)) {
     $YearDirectors{$title.$year}= $director;
   }
   $Directors{$title}= $director;
  }
  close(RIN); return(1);
}

sub Find_Director($;$)
{
  my ($title,$year)= @_;
  # Debug("Looking up director for $title\n");

  if (defined($year) && defined($YearDirectors{$title.$year})) {
    Debug("Found yeardirector $YearDirectors{$title.$year} for $title\n");
    return($YearDirectors{$title.$year});
  } else {
    if (defined($Directors{$title})) {
      Debug("Found director $Directors{$title} for $title\n");
      return($Directors{$title});
    }
  }
  return(undef);
}

1;
