#!/usr/bin/perl
# 
# FoxtelFetcher.pm: parse the data in web pages from www.foxtel.com.au
# and construct suitable data structures which can be converted into slice
# files for the TiVo.
#
# Due to legal reasons, I have had to cripple this fetcher so that it
# cannot grab data from the Foxtel website. All that is left is skeleton
# code which should be very similar to the code in all the other fetchers.
#
# Copyright (c) Warren Toomey, wkt@tuhs.org, 2002/2003/2004
#
#    $Revision: 3.33 $
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 0. This software must only be used in those parts of the world which
#    cannot obtain a program subscription service from TiVo, Inc.
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 3. All advertising materials mentioning features or use of this software
#    must display the following acknowledgement:
#      This product includes software developed by Warren Toomey.
# 4. Warren Toomey's name may not be used to endorse or promote products
#    derived from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY WARREN TOOMEY ``AS IS'' AND ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
# IN NO EVENT SHALL WARREN TOOMEY BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
# NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
package TiVo::FoxtelFetcher;
use strict;
use TiVo::ConfigFiles;
use TiVo::MiscUtil;
use HTTP::Lite;

BEGIN {
  use Exporter	 ();
  use vars	 qw($VERSION @ISA @EXPORT @EXPORT_OK %EXPORT_TAGS);

  $VERSION = do { my @r = (q$Revision: 3.33 $ =~ /\d+/g); sprintf "%d."."%02d" x $#r, @r }; # must be all one line, for MakeMaker

  @ISA	       = qw(Exporter);
  @EXPORT      = qw(Fetch_Foxtel_Channel);
  %EXPORT_TAGS = ( );
  @EXPORT_OK   = qw();
}

use vars	 @EXPORT_OK;
END { }	 # module clean-up code here (global destructor)


#####################################
###### Package-private Globals ######
#####################################

# Base URL used for Foxtel.
my ($FoxtelTopUrl)= "";
my ($FoxtelCookieUrl)= "";
my ($FoxtelDataUrl)=
		"";
my ($GlobalCookie)= "";

# Words to look for in program description to find the actors
my (@Try_cast)= ("Starring", "Hosted by", "Presented by",
		 "Cast:", "Guest stars", "Stars");

# Which days have already been parsed
my (%AlreadyDoneDay);

# The array of program data that is
# built up.
my (@FoxtelData, $progindex);

# Hashes to convert Foxtel genres to TiVo genres
my (%Genre) = (
  "Action" => "Action,ActionAdventureGroup",
  "Adult" => "Adult,DramaGroup",
  "Adventure" => "Action,ActionAdventureGroup",
  "Adventure / Action" => "Action,ActionAdventureGroup",
  "Animation / Cartoon" => "Animated,ChildrensGroup",
  "Arts" => "Art,ArtsMusicLiving",
  "Biography" => "Biography,DocumentaryGroup",
  "Box & Wrest" => "Wrestling",
  "Business / Finance" => "Business,NewsBusinessWeather",
  "Classical / Opera" => "Music,Opera,ArtsMusicLiving",
  "Comedy" => "Comedy,ComedyGroup",
  "Court Sports" => "Sports,SportsGroup",
  "Cricket" => "Sports,SportsGroup",
  "Dance / Techno" => "Dance,ArtsMusicLiving",
  "Documentaries" => "Documentary,DocumentaryGroup",
  "Drama" => "Drama,DramaGroup",
  "Drama / Romance" => "Drama,Romance,DramaGroup",
  "Educational" => "Educational,EducationalGroup",
  "Fantasy" => "Fantasy,SciFiFantasy",
  "Foreign Language Channels" => "International",
  "Game Show" => "GameShow,Daytime",
  "Golf" => "Golf,SportsGroup",
  "History" => "Historical,DocumentaryGroup",
  "Horror" => "Horror",
  "International" => "International",
  "Jazz / Blues" => "Music,ArtsMusicLiving",
  "Kids / Family" => "Family,ChildrensGroup",
  "Lifestyle / Doco" => "Outdoors,RealLife",
  "Light Ent" => "Variety",
  "Magazine / Discussion / Interview" => "Talkshow,Daytime",
  "Musical / Dance" => "Dance,Music,ArtsMusicLiving",
  "Mystery" => "Mystery,MysteryThriller",
  "Mystery / Crime" => "Mystery,MysteryThriller",
  "Natural World" => "Nature,DocumentaryGroup",
  "News & Other" => "News,NewsBusinessWeather",
  "People & Culture" => "Documentary,DocumentaryGroup",
  "Pop" => "Music,ArtsMusicLiving",
  "Public Affairs / Political" => "PublicAffairs,NewsBusinessWeather",
  "Racing" => "Horse,SportsGroup",
  "Rap & Hip Hop" => "Music,ArtsMusicLiving",
  "Reality" => "RealLife",
  "Reality / Fact" => "RealLife",
  "Rock" => "Music,ArtsMusicLiving",
  "Rugby League" => "Football,SportsGroup",
  "Rugby Union" => "Football,SportsGroup",
  "Sci Fi" => "ScienceFiction,SciFiFantasy",
  "Sci Fi / Fantasy" => "ScienceFiction,SciFiFantasy",
  "Science / Technology" => "Science,ScienceNatureTechnology",
  "Shopping" => "Shopping,Daytime",
  "Soccer" => "Soccer,SportsGroup",
  "Special Events" => "Specials",
  "Specials" => "Specials",
  "Sports" => "Sports,SportsGroup",
  "Talk Shows" => "Talkshow,Daytime",
  "Thriller / Suspense" => "Thriller",
  "Travel & Adventure" => "Travel,ActionAdventureGroup",
  "War / Western" => "War,Western",
);

# List of Foxtel guide data decode functions
my (%Decoder)= (
    "03" => \&FoxDecode03,
    "13" => \&FoxDecode03,
    "23" => \&FoxDecode23,
    "33" => \&FoxDecode23,
    "43" => \&FoxDecode43,
    "53" => \&FoxDecode43,
    "63" => \&FoxDecode63,
    "73" => \&FoxDecode63,
    "83" => \&FoxDecode83,
    "93" => \&FoxDecode83,
    "103" => \&FoxDecode10,
    "113" => \&FoxDecode10,
    "123" => \&FoxDecode12,
    "133" => \&FoxDecode12,
    "143" => \&FoxDecode14,
    "153" => \&FoxDecode14,
    "163" => \&FoxDecode16,
    "173" => \&FoxDecode16,
);

sub charAt($$)
{
 my ($str, $pos)= @_;

 return(undef) if ($pos<0);
 return(substr($str,$pos,1));
}

# Foxtel guide data decoder functions. If I knew
# who the author of these function were, I would
# be able to give them credit for writing them.
#



######################
###### Routines ######
######################

# Given a daynumber relative to today (0, 1, 2 etc.), return the
# dayname as a Foxtel string.
sub Get_Foxtel_Dayname($)
{
  my ($wanteday)= $_[0];
  my ($dayname);
  my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst);

  # We want a dd/mm/yy format.
  ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst)= Get_Daylist($wanteday);
  $mon++; $year%= 100;
  $dayname= sprintf("%02d/%02d/%02d", $mday, $mon, $year);
  return($dayname);
}

# Given a channel name, the relative start and
# end days, fetch the web data for this channel.
# Rehtml it and save it. Then parse the data
# into the intermediate format. Returns 1 on
# no errors, 0 if errors.
sub Fetch_Foxtel_Channel($$$)
{
  my ($chan,$snum,$enum)= @_;
  my ($result1,$result2)= (1,1);
  $result1=Fetch_and_Store_Data($chan,$snum,$enum);
  $result2=Parse_Data($chan,$snum,$enum);
  return($result1 && $result2);
}

sub Fetch_and_Store_Data($$$)
{
  my ($chan,$snum,$enum)= @_;
  my ($web_proxy, $webdelay, $wanteday);
  my ($html_file, $dayname, $daynum, $outfile);
  my ($result, $try_again, $try, $http, $req, $body);
  my (%args, $daystring, $viewstate, $httpget);

  # We start 1 day early so that we get the midnight to 6am data
  $result=1; $snum--;

  # See if we need a web proxy, and # of times to try again
  $web_proxy= $ConfigVar{Webproxy};
  $try_again= $ConfigVar{RepeatWebConnect};

  # Set the delay in seconds between web fetches
  $webdelay=0;
  if (defined($ConfigVar{WebDelay})) {
    $webdelay= int($ConfigVar{WebDelay});
  }

  # Do each day in turn
  for ($wanteday= $snum; $wanteday <= $enum; $wanteday++) {

    $dayname= Get_Dayname($wanteday);
    $daynum= Get_Daynumber($wanteday);

    # Skip if we have already generated the output file
    $outfile=	"$ConfigVar{Datadir}/$chan" . "_$dayname";
    if (-f $outfile) {
      Debug("Intermediate file $outfile exists, not fetching\n");
      next;
    }

    # Get the output filename
    $html_file= "$ConfigVar{Webdir}/$chan" . "_$dayname";

    # If the HTML file already exists, assume we have already run
    # this before and stored it in the cache.
    if ( -f $html_file) {
      Debug("Using cached data for channel $chan for $dayname\n");
    } else {
      Debug("Fetching programs for channel $chan for $dayname\n");

      $http = new HTTP::Lite;
      $httpget = new HTTP::Lite;
      if (defined($web_proxy)) {
	 $http->proxy($web_proxy);
	 $httpget->proxy($web_proxy);
      }

      # Get the day string as required by Foxtel
      $daystring= Get_Foxtel_Dayname($wanteday);

      # Try 1 or more times to get the file
      if (!defined($try_again)) { $try_again=1; }

      for ($try=$try_again;$try>0;$try--) {

        if ($try==0) {
	  print(STDERR "Unable to get data from Foxtel: $!\n");
	  $result=0; next;
        }
      }
      $body= FoxDecode($httpget->body());
      if (!defined($body)) {
	  print(STDERR "Unable to decode data from Foxtel\n");
	  $result=0; next;
      }

      Rehtml($body, $html_file, 0);
      WebLink($html_file, $chan, $dayname);
      sleep($webdelay);
    }
  }
  return($result);
}

# Obtain a whole web page. Find the section which
# is encoded, decode it and return it.
sub FoxDecode($)
{
}

sub Get_Foxtel_Viewstate()
{
}

# WebLink: see if other channels use the same URL and
# make a symlink to the file instead of fetching it.
sub WebLink($$$)
{
  my ($file, $origchan, $dayname)= @_;
  my ($chan, $html_file);

  foreach $chan (keys(%StationUrl)) {
    next if ($chan eq $origchan);
    if ($StationUrl{$chan} eq $StationUrl{$origchan}) {
      $html_file= "$ConfigVar{Webdir}/$chan" . "_$dayname";
      link($file, $html_file);
      Debug("Linked $file to $html_file as same URL\n");
    }
  }
}


# Given the channel, start day and end day, read the webfiles and
# convert into intermediate format. We now put all of one day, and
# only that day, into an output file.
# New overall strategy: read in each file and parse it.
# Build up a list of records with no duration. Once the
# files are read in, go back and deduce the duration
# by looking at the differences between start times.
# Finally, write the files out.
#
sub Parse_Data($$$)
{
  my ($chan,$snum,$enum)= @_;
  my (%IgnoreDay);
  my ($wanteday, $outfile, $result);
  my ($dayname, $daynum, $i, $pdindex, $start_index);

  $result=1; $progindex=0; undef(@FoxtelData);

  # Find out what days we must actually do, and parse the web files.
  for ($wanteday= $snum; $wanteday <= $enum; $wanteday++) {
    $dayname= Get_Dayname($wanteday);
    $outfile=	"$ConfigVar{Datadir}/$chan" . "_$dayname";

    # Skip if we have already generated the output file
    if (-f $outfile) {
      Debug("Intermediate file $outfile exists, not overwriting\n");
      $IgnoreDay{$wanteday}=1;
      # We insert a fictitious record with a 6am start anyway
      $FoxtelData[$progindex]{title}="Fictitious 6am record";
      $FoxtelData[$progindex]{daynum}= $FoxtelData[$progindex-1]{daynum};
      $FoxtelData[$progindex]{time}=21600; $progindex++;
      next;
    }

    # Build the list of records for the previous day and this day
    if (!defined($AlreadyDoneDay{$wanteday-1})) {
      $result &= Parse_Data_For_Day($chan,$wanteday-1);
    }
    $result &= Parse_Data_For_Day($chan,$wanteday);
    $AlreadyDoneDay{$wanteday}=1;
  }

  # Insert a final record to allow the last duration calculation
  $FoxtelData[$progindex]{title}="Fictitious 6am record";
  $FoxtelData[$progindex]{daynum}= $FoxtelData[$progindex-1]{daynum};
  $FoxtelData[$progindex]{time}=21600; $progindex++;

  # Calculate the duration for each program
  for ($i=0; $i< $progindex -1; $i++) {
    $FoxtelData[$i]{duration}=
	$FoxtelData[$i+1]{time} - $FoxtelData[$i]{time};
    $FoxtelData[$i]{duration}+= 86400 if ($FoxtelData[$i]{duration}<0);
    # Debug("$i: $FoxtelData[$i]{title}\n");
    # Debug("$FoxtelData[$i]{daynum}\n");
    # Debug("$FoxtelData[$i]{time} $FoxtelData[$i]{duration}\n\n");
  }

  # Now write the results out for each day where there is no data yet
  $start_index=0;
  for ($wanteday= $snum; $wanteday <= $enum; $wanteday++) {
    if (defined($IgnoreDay{$wanteday})) { next; }

    # Print out the relevant records for this daynum
    $dayname= Get_Dayname($wanteday);
    $daynum=  Get_Daynumber($wanteday); $pdindex=0; undef(@ProgramData);
    $outfile=	"$ConfigVar{Datadir}/$chan" . "_$dayname";

    for ($i=$start_index; $i< $progindex -1; $i++) {

      next if ($FoxtelData[$i]{daynum}<$daynum);
      if ($FoxtelData[$i]{daynum}>$daynum) {
	$start_index=$i; last;
      }
      Debug("Adding $FoxtelData[$i]{title} to $daynum\n");
      $ProgramData[$pdindex]= $FoxtelData[$i]; $pdindex++;
    }
    if ($pdindex!=0) {
      Debug("Writing file $outfile\n");
      $result &= Write_IntermediateFile($outfile);
    } else {
      print(STDERR "No Foxtel data for $chan on $dayname\n");
      $result=0;
    }
  }
  return($result);
}


# Parse a web file for only a single day.
sub Parse_Data_For_Day($$)
{
  my ($chan,$wanteday)= @_;
  my ($dayname, $daynum);
  my ($html_file);
  my ($this_time, $last_start_time);
  my ($hr, $min, $apm, $start_time);
  my ($title, $year, $tvrating, $showingbits);
  my ($i, @Advisory, @Actorlist, $description);
  my ($advisory, $actor, $genre);
  my ($episode_title, $g);
  my ($time_marker, $found_since_marker);

  $dayname= Get_Dayname($wanteday);
  $daynum=  Get_Daynumber($wanteday);
  $last_start_time=-1; $time_marker=-1;

  # Get the input filename
  $html_file= "$ConfigVar{Webdir}/$chan" . "_$dayname";

  unless (open(IN, $html_file)) {
    print(STDERR "Can't open downloaded web page $html_file: $!\n");
    return(0);
  }
  Debug("Parsing Foxtel data from $html_file\n");

  close(IN);

  if ($last_start_time == -1) {
    Debug("No Foxtel data in $html_file\n");
    # Unlink the file as it contained no data
    unlink($html_file);
    return(0);
  }

  # If we haven't seen any data since a time marker, put in
  # a record indicating that the station closed at that time
  if (($time_marker!=-1) && ($found_since_marker==0)) {
     Debug("Adding a CLOSE record for $daynum at time $time_marker\n");
     $FoxtelData[$progindex]{title}="Close";
     $FoxtelData[$progindex]{daynum}= $FoxtelData[$progindex-1]{daynum};
     $FoxtelData[$progindex]{time}=$time_marker;
     # Adjust in case it's on the next day
     if ($time_marker < $FoxtelData[$progindex-1]{time}) {
	$FoxtelData[$progindex]{daynum}++;
        Debug("CLOSE record daynum was bumped\n");
     }
     $progindex++;
  }

  return(1);
}

# Procedure find_the_cast:
# Use heuristics to try & work out who is in the cast.
# Look for CAST, Stars, Starring, UPPERCASE WORDS (for Channel Ten).
#
sub Find_The_Cast($$) {
    my ($channel, $dref)= @_;
    my ($description)= $$dref;
    my (@Actor, $trycast, $left, $right, $actor);

    foreach $trycast (@Try_cast) {
      #
      # If we find a match using any of the trycast words,
      # followed by a : or a space, then ....
      #
      if ($description =~ /(.*)$trycast(:\s*|\s+)/i) {
        $left=$1;
        #
        # Remove the stuff leading up to and including the trycast.
        # Then remove anything starting with a dot, a '(' and
        # words starting with lowercase letters.
        #
        $description=~ s/.*$trycast\s*:*\s*//i;
        $description=~ s/((\b[a-z].*|\..*|\(.*))//;

        if (defined($1)) { $right = $1; } else { $right = ""; }
        $$dref = $left . $right;	# Send new description back.
        # 
        # Now split on either commas or the patterns ' and '
        # and ' & '
        #
        push(@Actor,
      	LowercaseString(split(/,\s*|\s+and\s+|\s*&\s+/,$description)));
        return(@Actor);
      }
    }

    # No, luck try the uppercase method then :-)
    #
    #while ($description =~ /([A-Z][A-Z]+ [A-Z][A-Z]+)/g) {
    #  ($actor)= LowercaseString($1);
    #  Debug("Working on $actor\n");
    #  push(@Actor,$actor);
    #}
    return(@Actor);
}

1;
