#!/usr/bin/perl
# 
# Ratings.pm: Routines to load and use the ratings database.
#
# Copyright (c) Warren Toomey, wkt@tuhs.org, 2002/2003/2004, GPL license
#
#    $Revision: 3.2 $
#
# This file is distributed under the GNU Public License with the
# following additional limitations.
# 
# 1. This software must not be used in those parts of the world where
#    TiVo Inc. provides a subscription service.
# 2. This software must not be used for commercial purposes. You may 
#    not use this software to provide a "service" and charge for that
#    service.
#
# See Docs/COPYING for the GNU Public License details.
#
package TiVo::Ratings;
use strict;
use TiVo::ConfigFiles;
use TiVo::MiscUtil;

BEGIN {
  use Exporter	 ();
  use vars	 qw($VERSION @ISA @EXPORT @EXPORT_OK %EXPORT_TAGS);

  $VERSION = do { my @r = (q$Revision: 3.2 $ =~ /\d+/g); sprintf "%d."."%02d" x $#r, @r }; # must be all one line, for MakeMaker

  @ISA	       = qw(Exporter);
  @EXPORT      = qw(Read_Ratings Find_Rating);
  %EXPORT_TAGS = ();
  @EXPORT_OK   = qw();
}

use vars	 @EXPORT_OK;
END { }	 # module clean-up code here (global destructor)


#####################################
###### Package-private Globals ######
#####################################

my ($ratings_file)= "ratings.tivo.gz";
my (%Ratings, %YearRatings);

######################
###### Routines ######
######################

# Read_Ratings: read the ratings from the ratings.list.gz file.
sub Read_Ratings
{
  my ($Configdir);
  my ($title,$year,$rating);

  # Don't do anything if there is no ratings file
  $Configdir= $ConfigVar{"Configdir"};
  if (! -f "$Configdir/$ratings_file") {
    Debug("No ratings file $Configdir/$ratings_file!\n");
    return(1);
  }

  print("Loading ratings file...\n");
  unless (open(RIN, "zcat $Configdir/$ratings_file |")) {
    print(STDERR "Cannot open $Configdir/$ratings_file: $!\n");
    return(0);
  }

  # Walk the file obtaining per-movie ratings
  while (<RIN>) {
   chop;
   ($title,$rating,$year)= split(/\t/);
   if (defined($year)) {
     $YearRatings{$title.$year}= $rating;
   }
   $Ratings{$title}= $rating;
  }
  close(RIN); return(1);
}

sub Find_Rating($;$)
{
  my ($title,$year)= @_;
  # Debug("Looking up star rating for $title\n");

  if (defined($year) && defined($YearRatings{$title.$year})) {
    # Debug("Found yearrating $YearRatings{$title.$year} for $title\n");
    return($YearRatings{$title.$year});
  } else {
    if (defined($Ratings{$title})) {
      # Debug("Found rating $Ratings{$title} for $title\n");
      return($Ratings{$title});
    }
  }
  return(undef);
}

1;
