#!/usr/bin/perl
# 
# YahooFetcher.pm: parse the data in web pages from au.tv.yahoo.com
# and construct suitable data structures which can be converted into slice
# files for the TiVo.
#
# Due to legal reasons, I have had to cripple this fetcher so that it
# cannot grab data from the Foxtel website. All that is left is skeleton
# code which should be very similar to the code in all the other fetchers.
#
#
# Copyright (c) Warren Toomey, wkt@tuhs.org, 2002/2003/2004
#
#    $Revision: 1.13 $
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 0. This software must only be used in those parts of the world which
#    cannot obtain a program subscription service from TiVo, Inc.
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 3. All advertising materials mentioning features or use of this software
#    must display the following acknowledgement:
#      This product includes software developed by Warren Toomey.
# 4. Warren Toomey's name may not be used to endorse or promote products
#    derived from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY WARREN TOOMEY ``AS IS'' AND ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
# IN NO EVENT SHALL WARREN TOOMEY BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
# NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
package TiVo::YahooFetcher;
use strict;
use TiVo::ConfigFiles;
use TiVo::MiscUtil;
use HTTP::Lite;

BEGIN {
  use Exporter	 ();
  use vars	 qw($VERSION @ISA @EXPORT @EXPORT_OK %EXPORT_TAGS);

  $VERSION = do { my @r = (q$Revision: 1.13 $ =~ /\d+/g); sprintf "%d."."%02d" x $#r, @r }; # must be all one line, for MakeMaker

  @ISA	       = qw(Exporter);
  @EXPORT      = qw(Fetch_Yahoo_Channel);
  %EXPORT_TAGS = ( );
  @EXPORT_OK   = qw();
}

use vars	 @EXPORT_OK;
END { }	 # module clean-up code here (global destructor)


#####################################
###### Package-private Globals ######
#####################################

# Base URL used for Yahoo.
my ($YahooUrl)= "";
# URL for each program
my ($YahooProgUrl)= "";

# Words to look for in program description to find the actors
my (@Try_cast)= ("Starring", "Hosted by", "Presented by",
		 "Cast:", "Guest stars", "\. Stars");

# Which days have already been parsed
my (%AlreadyDoneDay);

# The hash of programs found on each web page
my (%Event);

# Yahoo to TiVo rating conversion
my (%YahooRating)= (
	" - G" =>	3,
	" - M" =>	6,
	" - MA" =>	6,
	" - PG" =>	4
);

# Yahoo to TiVo genre translation
my (%GenreTranslate) = (
	"Action" => "Action,ActionAdventureGroup",
	"Adventure" => "Action,ActionAdventureGroup",
	"Animation" => "Animated,ChildrensGroup",
	"Arts and Culture" => "Art,ArtsMusicLiving",
	"Biography" => "Biography,DocumentaryGroup",
	"Business and Finance" => "Business,NewsBusinessWeather",
	"Cartoon" => "Animated,ChildrensGroup",
	"Children" => "Animated,ChildrensGroup",
	"Comedy" => "Comedy,ComedyGroup",
	"Crime" => "Crime,DramaGroup",
	"Cult" => "Unknown",
	"Current Affairs" => "PublicAffairs,NewsBusinessWeather",
	"Documentary" => "Documentary,DocumentaryGroup",
	"Drama" => "Drama,DramaGroup",
	"Education" => "Educational,EducationalGroup",
	"Entertainment" => "Unknown",
	"Family" => "Family,ChildrensGroup",
	"Fantasy" => "Fantasy,SciFiFantasy",
	"Film-Noir" => "Unknown",
	"Foreign" => "International",
	"Game Show" => "GameShow,Daytime",
	"Historical" => "Historical,DocumentaryGroup",
	"Horror" => "Horror,Thriller",
	"Infotainment" => "Shopping",
	"Lifestyle" => "Outdoors,RealLife",
	"Murder/Mystery" => "Crime,Mystery,MysteryThriller",
	"Music" => "Music,ArtsMusicLiving",
	"Musical" => "Musical,ArtsMusicLiving",
	"Mystery" => "Mystery,MysteryThriller",
	"Nature" => "Nature,DocumentaryGroup",
	"News" => "News,NewsBusinessWeather",
	"Parliament" => "Horror",
	"Real Life" => "RealLife",
	"Religion" => "Religion",
	"Romance" => "Romance,DramaGroup",
	"Romantic Comedy" => "RomanticComedy,ComedyGroup",
	"Sci-Fi" => "ScienceFiction,SciFiFantasy",
	"Science and Technology" => "Science,ScienceNatureTechnology",
	"Shopping" => "Shopping",
	"Short" => "Unknown",
	"Sitcom" => "Situation,Daytime",
	"Soap Opera" => "SoapOpera,DramaGroup",
	"Sport" => "Sports,SportsGroup",
	"Suspense" => "Suspense,MysteryThriller",
	"Talk Show" => "Talkshow,Daytime",
	"Thriller" => "Thriller",
	"Travel" => "Travel",
	"Variety" => "Variety",
	"War" => "War,DocumentaryGroup",
	"Weather" => "Weather,NewsBusinessWeather",
	"Western" => "Western"
);



######################
###### Routines ######
######################

# Given a daynumber relative to today (0, 1, 2 etc.), return the
# dayname as a Austar string.
sub Get_Yahoo_Dayname($)
{
  my ($wanteday)= $_[0];
  my ($dayname);
  my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst);

  # We want a yyyy-mm-dd format
  ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst)= Get_Daylist($wanteday);
  $mon++; $year+= 1900;
  $dayname= sprintf("%04d-%d-%d", $year, $mon, $mday);
  return($dayname);
}

# Given a channel name, the relative start and
# end days, fetch the web data for this channel.
# Rehtml it and save it. Then parse the data
# into the intermediate format. Returns 1 on
# no errors, 0 if errors.
sub Fetch_Yahoo_Channel($$$)
{
  my ($chan,$snum,$enum)= @_;
  my ($result1,$result2)= (1,1);
  $result1=Fetch_and_Store_Data($chan,$snum,$enum);
  $result2=Parse_Data($chan,$snum,$enum);
  return($result1 && $result2);
}

# Fetch the main page for this channel and save it.
#
sub Fetch_and_Store_Data($$$)
{
  my ($chan,$snum,$enum)= @_;
  my ($web_proxy, $webdelay, $wanteday, $yahoodayname);
  my ($html_file, $dayname, $url, $outfile);
  my ($result, $try_again, $try, $http, $req, $body);

  $result=1;

  # See if we need a web proxy, and # of times to try again
  $web_proxy= $ConfigVar{Webproxy};
  $try_again= $ConfigVar{RepeatWebConnect};

  # Set the delay in seconds between web fetches
  $webdelay=0;
  if (defined($ConfigVar{WebDelay})) {
    $webdelay= int($ConfigVar{WebDelay});
  }

  # Do each day in turn
  for ($wanteday= $snum; $wanteday <= $enum; $wanteday++) {

    $yahoodayname= Get_Yahoo_Dayname($wanteday);
    $dayname= Get_Dayname($wanteday);

    # Skip if we have already generated the output file in Data/
    $outfile=	"$ConfigVar{Datadir}/$chan" . "_$dayname";
    if (-f $outfile) {
      Debug("Intermediate file $outfile exists, not fetching\n");
      next;
    }

    # Determine the Yahoo web page for this station and dayname

    # Get the output filename
    $html_file= "$ConfigVar{Webdir}/$chan" . "_$dayname";

    # If the HTML file already exists, assume we have already run
    # this before and stored it in the cache.
    if ( -f $html_file) {
      Debug("Using cached data for channel $chan for $dayname\n");
    } else {
      Debug("Fetching programs for channel $chan for $dayname\n");

      $http = new HTTP::Lite;
      if (defined($web_proxy)) { $http->proxy($web_proxy); }

      # Try 1 or more times to get the file
      if (!defined($try_again)) { $try_again=1; }

      for ($try=$try_again;$try>0;$try--) {
	Debug("Fetching $url\n");
        my($req) = $http->request($url) && last;
      }

      if ($try==0) {
	print(STDERR "Unable to get data from Yahoo: $!\n");
	$result=0; next;
      }

      # Now pass the returned document to rehtml to place 1 tag per line
      # and sleep afterwards
      $body= $http->body();
      Rehtml($body, $html_file, 0);
      $result=Extend_Data($chan,$wanteday);
      WebLink($html_file, $chan, $dayname);
      sleep($webdelay);
    }
  }
  return($result);
}

# Extend the data on a page by fetching the sub-pages,
# one for each program
#
sub Extend_Data($$)
{
  my ($chan,$wanteday)= @_;
  my ($web_proxy, $webdelay);
  my ($html_file, $dayname, $url, $already_extended);
  my ($result, $try_again, $try, $http, $req, $body);
  my ($event,$title,$rating,@Elist);

  $result=1;

  # See if we need a web proxy, and # of times to try again
  $web_proxy= $ConfigVar{Webproxy};
  $try_again= $ConfigVar{RepeatWebConnect};

  # Set the delay in seconds between web fetches
  $webdelay=0;
  if (defined($ConfigVar{WebDelay})) {
    $webdelay= int($ConfigVar{WebDelay});
  }

  $dayname= Get_Dayname($wanteday);
  $already_extended=0;

  # Get the input and output filename
  $html_file= "$ConfigVar{Webdir}/$chan" . "_$dayname";

  if (! (-f $html_file)) {
    print(STDERR "Unable to extend data from Yahoo file $html_file: $!\n");
    $result=0; next;
  }
  unless (open(IN, $html_file)) {
    print(STDERR "Can't open downloaded web page $html_file: $!\n");
    return(0);
  }

  # We want to cache the event number, the title and the rating
  while (<IN>) {
  }
  close(IN);
  if ($already_extended) {
     Debug("Skipping Yahoo file $html_file: already was extended\n"); next;
  }

  # Now we fetch all the events that are outstanding
  Debug("Fetching events for Yahoo file $html_file\n") if (defined(@Elist));
  foreach $event (@Elist) {
    next if (defined($Event{$event}{done}));
  }
  return($result);
}

# WebLink: see if other channels use the same URL and
# make a symlink to the file instead of fetching it.
sub WebLink($$$)
{
  my ($file, $origchan, $dayname)= @_;
  my ($chan, $html_file);

  foreach $chan (keys(%StationUrl)) {
    next if ($chan eq $origchan);
    if ($StationUrl{$chan} eq $StationUrl{$origchan}) {
      $html_file= "$ConfigVar{Webdir}/$chan" . "_$dayname";
      link($file, $html_file);
      Debug("Linked $file to $html_file as same URL\n");
    }
  }
}


# Given the channel, start day and end day, read the webfiles and
# convert into intermediate format. We now put all of one day, and
# only that day, into an output file.
# New overall strategy: read in each file and parse it.
# Build up a list of records with no duration. Once the
# files are read in, go back and deduce the duration
# by looking at the differences between start times.
# Finally, write the files out.
#
sub Parse_Data($$$)
{
  my ($chan,$snum,$enum)= @_;
  my ($wanteday, $dayname, $outfile, $infile, $progindex, @Actorlist);
  my ($event, $timeline, $title, $episode, $yagenre, $description);
  my ($duration, $hr, $min, $apm, $start_time, $tvrating, $i, $actor);
  my ($genre, $daynum, $found_am, $live, $premiere, $stuff, $bits);
  my ($result)=1;

  # Find out what days we must actually do, and parse the web files.
  for ($wanteday= $snum; $wanteday <= $enum; $wanteday++) {
    $dayname= Get_Dayname($wanteday);
    $daynum= Get_Daynumber($wanteday);
    # Get the input filename
    $infile= "$ConfigVar{Webdir}/$chan" . "_$dayname";
    $outfile=	"$ConfigVar{Datadir}/$chan" . "_$dayname";

    # Skip if we have already generated the output file
    if (-f $outfile) {
      Debug("Intermediate file $outfile exists, not overwriting\n");
      next;
    }
    unless (open(IN, $infile)) {
      print(STDERR "Can't open downloaded web page $infile: $!\n");
      return(0);
    }
    Debug("Parsing Yahoo data from $infile\n");
    $progindex=0; $found_am=0; undef(@ProgramData);

    # Time to extract the data for each day
    while (<IN>) {
    }
    close(IN);

    if ($progindex!=0) {
      $result &= Write_IntermediateFile($outfile);
    } else {
      Debug("No Yahoo data for $daynum $chan $dayname\n");
      print(STDERR "No Yahoo data for $chan on $dayname\n");
      unlink($infile); $result=0;
    }
  }
  return($result);
}

# Procedure find_the_cast:
# Use heuristics to try & work out who is in the cast.
# Look for CAST, Stars, Starring, UPPERCASE WORDS (for Channel Ten).
#
sub Find_The_Cast($$) {
    my ($channel, $dref)= @_;
    my ($description)= $$dref;
    my (@Actor, $trycast, $left, $right, $actor);

    foreach $trycast (@Try_cast) {
      #
      # If we find a match using any of the trycast words,
      # followed by a : or a space, then ....
      #
      if ($description =~ /(.*)$trycast(:\s*|\s+)/i) {
        $left=$1;
        #
        # Remove the stuff leading up to and including the trycast.
        # Then remove anything starting with a dot, a '(' and
        # words starting with lowercase letters.
        #
        $description=~ s/.*$trycast\s*:*\s*//i;
        $description=~ s/((\b[a-z].*|\..*|\(.*))//;

        if (defined($1)) { $right = $1; } else { $right = ""; }
        $$dref = $left . $right;        # Send new description back.
        # 
        # Now split on either commas or the patterns ' and '
        # and ' & '
        #
        push(@Actor,
        LowercaseString(split(/,\s*|\s+and\s+|\s*&\s+/,$description)));
        return(@Actor);
      }
    }

    # No, luck try the uppercase method then :-)
    #
    #while ($description =~ /([A-Z][A-Z]+ [A-Z][A-Z]+)/g) {
    #  ($actor)= LowercaseString($1);
    #  Debug("Working on $actor\n");
    #  push(@Actor,$actor);
    #}
    return(@Actor);
}


1;
