#!/usr/bin/perl
#
# Run this to get a week's worth of data.
#
# Set the confile file below. This allows you to have
# different doit scripts with completely different configs.

$config= ".guiderc";
$cmd= "all";
$stopflag= "";

select STDERR; $| = 1;      # make unbuffered
select STDOUT; $| = 1;      # make unbuffered


if ($#ARGV >= 0)
{
	if ($ARGV[0] eq "-n")
	{
		$stopflag= "-n";
		shift;
	}
	if ($ARGV[0] eq "-c")
	{
		$config= $ARGV[1];
		shift;
		shift;
	}
	$cmd= $ARGV[0] if ($#ARGV >= 0);
}

# Ensure we have at least a command.
if ($cmd eq "")
{
	print "Usage: doit.pl [-n] [-c configfile] [<Command>]\n";
	print "   -n: stops writing of numbers in the make phase\n";
	print "   -c: use 'configfile' instead of default '.guiderc'\n";
	print "   Command is one of the following (default = 'all'):\n";
	print "      fetch: gets the web pages\n";
	print "      search: searches yahoo for unknown genres\n";
	print "      add: augments the intermediate data\n";
	print "      make: makes the slice file\n";
	print "      both: does the fetch and make phases\n";
	print "      all: does all four phases\n";
	exit(1);
}

# Get day of week.
$day= (localtime)[6];
if ($day == 4) { $daylist= "All 1 7"; }		# Thu
elsif ($day == 5) { $daylist= "All 1 7"; }		# Fri
elsif ($day == 6) { $daylist= "All 1 7"; }		# Sat
elsif ($day == 0) { $daylist= "All 1 7"; }		# Sun
elsif ($day == 1) { $daylist= "All 1 7"; }		# Mon
elsif ($day == 2) { $daylist= "All 1 7"; }		# Tue
elsif ($day == 3) { $daylist= "All 1 7"; }		# Wed

if ($cmd eq "fetch" || $cmd eq "both" || $cmd eq "all")
{
   print "About to fetch and save 1 week of data from the Web\n";
   print `perl fetch_data -c $config $daylist`;
   print "\n" if ($cmd eq "both" || $cmd eq "all");
}

if ($cmd eq "search" || $cmd eq "all")
{
   print "Now searching yahoo for new and unknown programs\n";
   print `perl genre_search -c $config $daylist`;
   print "\n" if ($cmd eq "all");
}

if ($cmd eq "add" || $cmd eq "all")
{
   print "Now augmenting the intermediate data\n";
   print `perl add_data -c $config $daylist`;
   print "\n" if ($cmd eq "all");
}

if ($cmd eq "make" || $cmd eq "both" || $cmd eq "all")
{
   print "Now generating the new slice file\n";
   print `perl make_slice $stopflag -c $config $daylist`;
   print "\n" if ($cmd eq "both" || $cmd eq "all");
}

if ($^O eq "MSWin32")
{
	print "Done. Press Enter to continue...";
	getc();
}
exit(0);
